"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cdk_1 = require("@aws-cdk/cdk");
const aws_cdk_1 = require("aws-cdk");
const aws_sdk_1 = require("aws-sdk");
const lib = require("../lib");
const seagull_app_1 = require("../seagull_app");
const set_aws_credentials_1 = require("../set_aws_credentials");
class SeagullPipeline {
    constructor(props) {
        this.appPath = props.appPath;
        this.branch = props.branch;
        this.mode = props.mode;
        this.profile = props.profile;
        this.owner = props.owner;
        this.region = props.region;
        this.repository = props.repository;
        this.ssmParam = props.ssmParameter;
        this.githubToken = props.githubToken;
        this.actions = [
            'cloudformation:*',
            'cloudfront:*',
            'iam:*',
            's3:*',
            'apigateway:*',
            'lambda:*',
            'logs:*',
            'events:*',
        ];
        this.install = ['npm i -g npm', 'npm ci'];
        this.build = ['npm run build'];
        this.postBuild = [
            'npm run test',
            `BRANCH_NAME=${this.branch} DEPLOY_MODE=${this.mode} npm run deploy`,
        ];
    }
    createPipeline() {
        return __awaiter(this, void 0, void 0, function* () {
            set_aws_credentials_1.setCredsByProfile(this.profile);
            const isTest = this.mode === 'test';
            const suffix = `${isTest ? `-${this.branch}-test` : ''}-ci`;
            const pkgJson = require(`${this.appPath}/package.json`);
            const name = `${pkgJson.name}${suffix}`;
            const sdk = new aws_cdk_1.SDK({});
            const account = yield sdk.defaultAccount();
            const projectName = name;
            const stackProps = { env: { account, region: this.region } };
            const props = { projectName, sdk, stackProps };
            const actions = this.actions;
            const pipelineApp = new seagull_app_1.SeagullApp(props);
            const stack = pipelineApp.stack;
            const principal = 'codebuild.amazonaws.com';
            const role = stack.addIAMRole('role', principal, actions);
            const pipeline = stack.addPipeline('pipeline');
            const ssmSecret = this.ssmParam ? getSSMSecret(this.ssmParam) : undefined;
            const gitDataProps = {
                branch: this.branch,
                owner: this.owner,
                pkg: pkgJson,
                repo: this.repository,
                secretParameter: yield ssmSecret,
                token: this.githubToken,
            };
            const gitData = lib.getGitData(gitDataProps);
            const sourceConfig = {
                atIndex: 0,
                branch: gitData.branch,
                oauthToken: gitData.secret,
                owner: gitData.owner,
                pipeline,
                repo: gitData.repo,
            };
            stack.addSourceStage('source', sourceConfig);
            const buildConfig = {
                atIndex: 1,
                build: this.build,
                install: this.install,
                pipeline,
                postBuild: this.postBuild,
                role,
            };
            stack.addBuildStage('build', buildConfig);
            return pipelineApp;
        });
    }
    deployPipeline() {
        return __awaiter(this, void 0, void 0, function* () {
            const pipeline = yield this.createPipeline();
            yield pipeline.deployStack();
        });
    }
}
exports.SeagullPipeline = SeagullPipeline;
function getSSMSecret(parameter) {
    return __awaiter(this, void 0, void 0, function* () {
        const ssm = new aws_sdk_1.SSM({ credentials: aws_sdk_1.config.credentials });
        const param = { Name: parameter, WithDecryption: true };
        const ssmParam = yield ssm.getParameter(param).promise();
        const value = ssmParam && ssmParam.Parameter && ssmParam.Parameter.Value;
        return value ? new cdk_1.Secret(value) : undefined;
    });
}
//# sourceMappingURL=seagull_pipeline.js.map